@@ -195,6 +195,7 @@ class MBIODevice(object):
         self._vendor=None
         self._model=None
         self._version=None
+        self._firmware=None
         self._error=False
         self._conditionalRegisterWriteCount=0
         self._conditionalWrite=False
@@ -512,6 +513,10 @@ class MBIODevice(object):
     def version(self):
         return self._version
 
+    @property
+    def firmware(self):
+        return self._firmware
+
     def readDiscreteInputs(self, start, count=1):
         try:
             self.gateway.checkIdleAfterSend()
@@ -972,6 +977,11 @@ class MBIODevice(object):
                         self._version=data['version']
                 except:
                     pass
+                try:
+                    if data['firmware']:
+                        self._firmware=data['firmware']
+                except:
+                    pass
 
             if self.ping():
                 self.setState(self.STATE_POWERON)
@@ -1113,6 +1123,13 @@ class MBIODevice(object):
     def __getitem__(self, key):
         return self.values[key]
 
+    def isRebootPossible(self):
+        return False
+
+    def reboot(self):
+        self.logger.error('%s: reboot not implemented!')
+        return False
+
 
 class MBIODeviceGeneric(MBIODevice):
     def onInit(self):
