@@ -564,7 +564,13 @@ class MBIODeviceDigimatSIO(MBIODevice):
         r=self.readInputRegisters(98, 1)
         if r and (r[0] & 0xFF00) == 0xA500:
             data={'version': str(r[0] & 0xff),
-                  'model': 'SIO'}
+                  'model': 'SIO',
+                  'firmware': None}
+
+            # read firmware revision (not available before 0.9.11)
+            r=self.readInputRegisters(116, 3)
+            if r:
+                data['firmware']=f'{r[0]}.{r[1]}.{r[2]}'
 
             r=self.readInputRegisters(104, 3)
             if r:
@@ -578,6 +584,21 @@ class MBIODeviceDigimatSIO(MBIODevice):
 
         return None
 
+    def configurator(self):
+        configurator=SIOConfigurator(self.gateway.host)
+        return configurator
+
+    def isRebootPossible(self):
+        return True
+
+    def reboot(self):
+        # FIXME: to be implemented through modbus
+        c=self.configurator()
+        if c.connect():
+            c.reboot()
+            return True
+        return False
+
 
 class SIOConfigurator(object):
     def __init__(self, host, token=None, logger=None):
