@@ -39,34 +39,40 @@ class GSheet(object):
         self._timeout=timeout
         self._client=None
         self._timeoutApi=0
-        self._delayApi=0
+        self._bufferWrite={}
 
         self._spreadsheet=None
         self._idspreadsheet=None
-        self._sheets=None
-        self._sheetsById=None
-        self._sheetsByIndex=None
-        self._sheetsByName=None
+        self._worksheets=None
+        self._worksheetsById=None
+        self._worksheetsByIndex=None
+        self._worksheetsByName=None
         self.resetCache()
 
     def resetCache(self):
-        self._sheets=[]
-        self._sheetsById={}
-        self._sheetsByIndex={}
-        self._sheetsByName={}
+        self._worksheets=[]
+        self._worksheetsById={}
+        self._worksheetsByIndex={}
+        self._worksheetsByName={}
 
-    def sheet(self, key=0) -> gspread.worksheet.Worksheet:
-        if self._spreadsheet and self._sheets:
+    def worksheet(self, key=0) -> gspread.worksheet.Worksheet:
+        if self._spreadsheet and self._worksheets:
             try:
-                return self._sheetsById[key]
+                return self._worksheetsById[key]
             except:
                 pass
             try:
-                return self._sheetsByIndex[key]
+                return self._worksheetsByIndex[key]
             except:
                 pass
             try:
-                return self._sheetsByName[key.lower()]
+                return self._worksheetsByName[key.lower()]
+            except:
+                pass
+            try:
+                # "sheet!a1"
+                name=key.split('!')[0]
+                return self._worksheetsByName[name.lower()]
             except:
                 pass
 
@@ -76,6 +82,7 @@ class GSheet(object):
         wait=(2 ** n) + random.random()
         time.sleep(wait)
 
+    # Warning: returns None is a sheetname doesn't exists
     def batchget(self, addresses, retries=5):
         for n in range(retries):
             try:
@@ -101,10 +108,10 @@ class GSheet(object):
                 break
         return None
 
-    def get(self, sheet, addresses, retries=5):
+    def get(self, sheetname: str, addresses, retries=5):
         for n in range(retries):
             try:
-                worksheet=self.sheet(sheet)
+                worksheet=self.worksheet(sheetname)
                 return worksheet.batch_get(addresses)
             except gspread.exceptions.APIError as e:
                 if "429" in str(e):
@@ -117,7 +124,8 @@ class GSheet(object):
         for n in range(retries):
             try:
                 spreadsheet=self.spreadsheet()
-                spreadsheet.values_batch_update(data)
+                body={'value_input_option': 'USER_ENTERED', 'data': data}
+                spreadsheet.values_batch_update(body)
                 return True
             except gspread.exceptions.APIError as e:
                 if "429" in str(e):
@@ -126,10 +134,47 @@ class GSheet(object):
                 pass
         return False
 
-    def set(self, sheet, data, retries=5):
+    def resetBufferedWrite(self):
+        self._bufferWrite={}
+
+    def isBufferedWrite(self):
+        if self._bufferWrite:
+            return True
+        return False
+
+    def bufferedWrite(self, address, value) -> bool:
+        if self._bufferWrite is None:
+            self._bufferWrite={}
+
+        if address and value is not None:
+            address=address.strip()
+            if '!' in address:
+                sheetname=address.split('!')[0]
+                if not self.worksheet(sheetname):
+                    return False
+            self._bufferWrite[address]=value
+            return True
+        return False
+
+    def commit(self):
+        if self.isBufferedWrite():
+            data=[]
+            for address in self._bufferWrite.keys():
+                value=self._bufferWrite[address]
+                if isinstance(value, (list, tuple)):
+                    data.append({'range': address, 'values': [value]})
+                else:
+                    data.append({'range': address, 'values': [[value]]})
+
+            result=self.batchset(data)
+            self.resetBufferedWrite()
+            return result
+        return True
+
+    def set(self, sheetname: str, data, retries=5):
         for n in range(retries):
             try:
-                worksheet=self.sheet(sheet)
+                worksheet=self.sheet(sheetname)
                 worksheet.batch_update(data)
                 return True
             except gspread.exceptions.APIError as e:
@@ -140,32 +185,34 @@ class GSheet(object):
         return False
 
     def __getitem__(self, key):
-        return self.sheet(key)
+        return self.worksheet(key)
 
-    def addSheet(self, sheet: gspread.worksheet.Worksheet) -> gspread.worksheet.Worksheet:
-        if sheet:
+    def addWorksheet(self, worksheet: gspread.worksheet.Worksheet) -> gspread.worksheet.Worksheet:
+        if worksheet:
             try:
-                sid=sheet.id
-                if not self.sheet(sid):
-                    self._sheets.append(sheet)
-                    self._sheetsById[sid]=sheet
-                    self._sheetsByIndex[sheet.index]=sheet
-                    self._sheetsByName[sheet.title.lower()]=sheet
+                sid=worksheet.id
+                if not self.worksheet(sid):
+                    self._worksheets.append(worksheet)
+                    self._worksheetsById[sid]=worksheet
+                    self._worksheetsByIndex[worksheet.index]=worksheet
+                    self._worksheetsByName[worksheet.title.lower()]=worksheet
             except:
+                self.logger.exception('x')
                 pass
-        return sheet
+        return worksheet
 
-    def retrieveSheets(self):
+    def retrieveWorksheets(self):
         self.resetCache()
         if self._spreadsheet:
             try:
-                sheets=self._spreadsheet.worksheets()
-                if sheets:
-                    for sheet in sheets:
-                        self.addSheet(sheet)
+                worksheets=self._spreadsheet.worksheets()
+                if worksheets:
+                    for worksheet in worksheets:
+                        self.addWorksheet(worksheet)
             except:
+                self.logger.exception('x')
                 pass
-        return self._sheets
+        return self._worksheets
 
     def client(self) -> gspread.client.Client:
         if self._client:
@@ -221,27 +268,27 @@ class GSheet(object):
                     if spreadsheet:
                         self._spreadsheet=spreadsheet
                         self._idspreadsheet=key
-                        self.retrieveSheets()
+                        self.retrieveWorksheets()
                 pass
         except:
             pass
 
         return self._spreadsheet
 
-    def cell(self, sheet: gspread.worksheet.Worksheet, address) -> gspread.cell.Cell:
+    def cell(self, worksheet: gspread.worksheet.Worksheet, address) -> gspread.cell.Cell:
         try:
-            if sheet and address:
-                return sheet.acell(address)
+            if worksheet and address:
+                return worksheet.acell(address)
         except:
             pass
 
     def __repr__(self):
         try:
-            return '%s(%s, %d sheets)' % (self.__class__.__name__, self._spreadsheet.title, len(self._sheets))
+            return '%s(%s, %d sheets)' % (self.__class__.__name__, self._spreadsheet.title, len(self._worksheets))
         except:
             pass
-        if self._sheets:
-            return '%s(%s)' % (self.__class__.__name__, len(self._sheets))
+        if self._worksheets:
+            return '%s(%s)' % (self.__class__.__name__, len(self._worksheets))
         return '%s()' % (self.__class__.__name__)
 
 
@@ -256,9 +303,8 @@ class MBIOTaskGSheet(MBIOTask):
         self.config.set('timeout', 10)
         self.config.set('id')
         self._timeoutRefresh=0
-        self._timeoutSync=0
+        self._timeoutCommit=0
         self._timeoutRefreshNamedRanges=0
-        self.valueDigital('comerr', default=False)
         self._sheets={}
         self._cells={}
         self._addresses={}
@@ -273,44 +319,58 @@ class MBIOTaskGSheet(MBIOTask):
             return cell
         return None
 
-    # def cellValue(self, name):
-        # cell=self.cell(name)
-        # if cell:
-            # return cell['value']
-        # return None
-
-
-    # TODO: pas de override pour AI --> add IMPORT item (cell->override-item) without mbio-value (must be writable)
-    # pas de expression dans AO (uniquement writable) --> add EXPORT item (expression->cell) without mbio-value
-
     def isA1Notation(self, address: str) -> bool:
-        return re.fullmatch(r"[A-Z]{1,3}+[1-9][0-9]*", address, re.IGNORECASE) is not None
+        if address:
+            return re.fullmatch(r"[A-Z]{1,3}+[1-9][0-9]*", address, re.IGNORECASE) is not None
+        return False
 
-    def declareAddress(self, sheet, cell):
-        if sheet and cell:
+    def normalizeAddress(self, sheet, address) -> str:
+        if sheet and address:
+            address=self.addressFromNamedRange(address)
             # allows for sheet!a1 notation or pure named field single string
-            if self.isA1Notation(cell):
-                address="%s!%s"  % (sheet['name'], cell)
-            else:
-                # named field
-                address=cell
+            if address is not None:
+                if self.isA1Notation(address):
+                    address="%s!%s"  % (sheet['name'], address)
+                address=address.lower()
+        return address
 
-            address=address.lower()
-            if not address in self._addresses:
+    def addressFromNamedRange(self, name):
+        try:
+            address=self._namedRanges[name.lower()]
+            if address is not None:
+                return address
+        except:
+            pass
+        return name
+
+    def worksheetFromAddress(self, address) -> gspread.worksheet.Worksheet:
+        gs=self.gsheet()
+        address=self.addressFromNamedRange(address)
+        return gs.worksheet(address)
+
+    def declareAddress(self, sheet, address):
+        self.logger.warning("DECLARE %s" % address)
+        address=self.normalizeAddress(sheet, address)
+        self.logger.warning("  normalized to %s" % address)
+        if address and not address in self._addresses:
                 self._addresses[address]=None
-            return address
-        return None
+        return address
+
+    def getAdressValue(self, sheet, address):
+        address=self.normalizeAddress(sheet, address)
+        return self._addresses.get(address)
 
     def loadCell(self, sheet, xml: XMLConfig, writable=True, digital=False, variables=None):
         if sheet:
             name=xml.get('name')
             name=self.replaceVariables(name, variables)
+            vname='%s_%s' % (sheet['alias'], name)
 
             address=xml.get('cell') or xml.get('address')
             address=self.replaceVariables(address, variables)
 
-            if name and not self.cell(name) and address:
-                self.declareAddress(sheet, address)
+            if name and not self.cell(vname) and address:
+                xaddress=self.declareAddress(sheet, address)
                 retain=False
                 storeunit=False
                 if writable:
@@ -318,7 +378,6 @@ class MBIOTaskGSheet(MBIOTask):
                 else:
                     retain=xml.getBool('retain', True)
 
-                vname='%s_%s' % (sheet['alias'], name)
                 default=xml.getFloat('default')
                 if digital:
                     value=self.valueDigital(vname, writable=writable, default=default, commissionable=True)
@@ -329,43 +388,43 @@ class MBIOTaskGSheet(MBIOTask):
 
                 if retain:
                     try:
-                        v=self.pickleRead('cell-%s-retain' % name)
+                        v=self.pickleRead('%s-cell-retain' % vname)
                         if v is not None:
                             value.updateValue(v)
                     except:
                         pass
 
-                xaddress=address
-                if self.isA1Notation(address):
-                    xaddress='%s!%s' % (sheet['name'], address)
-                data={'name': name, 'sheet': sheet['name'],
+                data={'name': vname, 'sheet': sheet['name'],
                       'address': address, 'xaddress': xaddress,
                       'writable': writable, 'retain': retain,
                       'value': value}
 
                 if writable:
-                    self._sheets[sheet['name']]['writablecells'].append(name)
+                    self._sheets[sheet['name']]['writablecells'].append(vname)
                     data['storeunit']=storeunit
                     # support for value expression calculations
                     self.loadValueExpression(value, xml, variables)
 
-                self.logger.debug('Declaring GSheet(%s) CELL %s' % (self.config.id, name))
-                self._cells[name]=data
-
-                self._sheets[sheet['name']]['cells'].append(name)
+                self.logger.debug('Declaring GSheet(%s) CELL %s' % (self.name, vname))
+                self._cells[vname]=data
+                self._sheets[sheet['name']]['cells'].append(vname)
 
                 return data
         return None
 
     def loadSheet(self, name, xml: XMLConfig):
+        if name:
+            name=name.strip()
         sheet=self._sheets.get(name)
         if xml is not None and sheet is None:
             alias=xml.get('alias', name)
-            self.logger.info('Declaring GSheet(%s-%s) SHEET %s (alias %s)' % (self.name, self.config.id, name, alias))
+            self.logger.info('Declaring GSheet(%s) SHEET %s (alias %s)' % (self.name, name, alias))
             sheet={'name': name, 'alias': alias, 'cells': [], 'writablecells': [], 'dumps': [], 'imports': [], 'exports': []}
             self._sheets[name]=sheet
 
-            myvariables={'gsheet': self.name, 'sheetname': sheet['name'], 'sheetalias': sheet['alias']}
+            myvariables={'gsheet': self.name,
+                         'sheetname': sheet['name'], 'name': sheet['name'],
+                         'sheetalias': sheet['alias'], 'alias': sheet['alias']}
 
             items=xml.children('ai')
             if items:
@@ -388,11 +447,13 @@ class MBIOTaskGSheet(MBIOTask):
             if items:
                 for item in items:
                     cell=item.get('cell')
+                    cell=self.replaceVariables(cell, myvariables)
                     key=item.get('key')
+                    key=self.replaceVariables(key, myvariables)
                     if cell and key:
                         refresh=item.getInt('refresh', 15, vmin=5)
                         data={'cell': cell, 'key': key, 'refresh': refresh, 'timeout': 0}
-                        self.logger.info('Declaring GSheet(%s-%s) SHEET %s (alias %s) DUMP [%s]->%s' % (self.name, self.config.id, name, alias, key, cell))
+                        self.logger.info('Declaring GSheet(%s) SHEET %s (alias %s) DUMP [%s]->%s' % (self.name, name, alias, key, cell))
                         sheet['dumps'].append(data)
 
             items=xml.children('import')
@@ -402,10 +463,9 @@ class MBIOTaskGSheet(MBIOTask):
                     key=item.get('key')
                     key=self.replaceVariables(key, myvariables)
                     if cell and key:
-                        self.declareAddress(sheet, cell)
-                        refresh=item.getInt('refresh', 15, vmin=5)
-                        data={'cell': cell, 'key': key}
-                        self.logger.info('Declaring GSheet(%s-%s) SHEET %s (alias %s) IMPORT %s->%s' % (self.name, self.config.id, name, alias, cell, key))
+                        xaddress=self.declareAddress(sheet, cell)
+                        data={'cell': cell, 'xaddress': xaddress, 'key': key}
+                        self.logger.info('Declaring GSheet(%s) SHEET %s (alias %s) IMPORT %s->%s' % (self.name, name, alias, cell, key))
                         sheet['imports'].append(data)
 
             items=xml.children('export')
@@ -415,10 +475,10 @@ class MBIOTaskGSheet(MBIOTask):
                     key=self.replaceVariables(key, myvariables)
                     cell=item.get('cell')
                     if key and cell:
-                        refresh=item.getInt('refresh', 15, vmin=5)
+                        xaddress=self.declareAddress(sheet, cell)
                         storeunit=item.getBool('storeunit', False)
-                        data={'key': key, 'cell': cell, 'storeunit': storeunit}
-                        self.logger.info('Declaring GSheet(%s-%s) SHEET %s (alias %s) EXPORT %s->%s' % (self.name, self.config.id, name, alias, key, cell))
+                        data={'key': key, 'cell': cell, 'xaddress': xaddress, 'storeunit': storeunit}
+                        self.logger.info('Declaring GSheet(%s) SHEET %s (alias %s) EXPORT %s->%s' % (self.name, name, alias, key, cell))
                         sheet['exports'].append(data)
 
     def onLoad(self, xml: XMLConfig):
@@ -436,8 +496,9 @@ class MBIOTaskGSheet(MBIOTask):
         items=xml.children('sheet')
         if items:
             for item in items:
-                name=item.get('name', 0)
-                self.loadSheet(name, item)
+                names=item.get('name', 0).split(',')
+                for name in names:
+                    self.loadSheet(name, item)
 
     def credentials(self):
         fname=os.path.join(self.getMBIO().rootPath() or '.', self.config.credentials)
@@ -476,15 +537,15 @@ class MBIOTaskGSheet(MBIOTask):
         return self._gsheet
 
     def poweron(self):
-        self.logger.info('Retrieving GSheet(%s-%s)' % (self.name, self.config.id))
+        self.logger.info('Retrieving GSheet(%s)' % (self.name))
         gs=self.gsheet(True)
         if gs:
             spreadsheet=gs.spreadsheet(self.config.id)
             if spreadsheet:
-                self.logger.debug('Connected to GSheet(%s-%s) %s' % (self.name, self.config.id, gs))
+                self.logger.debug('Connected to GSheet(%s) %s' % (self.name, gs))
                 return True
 
-        self.logger.error('Unable to retrieve GSheet(%s-%s)' % (self.name, self.config.id))
+        self.logger.error('Unable to retrieve GSheet(%s)' % (self.name))
         return False
 
     def poweroff(self):
@@ -505,23 +566,8 @@ class MBIOTaskGSheet(MBIOTask):
             pass
         return None
 
-    def addressFromNamedRange(self, name):
-        try:
-            return self._namedRanges[name.lower()]
-        except:
-            pass
-        return name
 
-    def col_to_a1(self, col_idx0: int) -> str:
-        # 0 -> A, 25 -> Z, 26 -> AA ...
-        col = ""
-        n = col_idx0 + 1
-        while n:
-            n, r = divmod(n - 1, 26)
-            col = chr(65 + r) + col
-        return col
-
-    def gridRangeToCellAddress(self, sheet, gr):
+    def gridRangeToCellAddress(self, worksheet: gspread.worksheet.Worksheet, gr):
         try:
             # sheetId is implicitely 0 if not specified!
             sC = gr.get("startColumnIndex")
@@ -535,23 +581,25 @@ class MBIOTaskGSheet(MBIOTask):
             if eR is None: eR = sR + 1
             if eC is None: eC = sC + 1
 
-            def cell(r0, c0):
-                return f"{self.col_to_a1(c0)}{r0+1}"
+            # range indexes are 0 based, while row/col addresses are 1 based
+            def cellAddress(r0, c0):
+                return self.rowcol_to_a1(r0+1, c0+1).lower()
 
-            start = cell(sR, sC)
-            end   = cell(eR - 1, eC - 1)
+            start = cellAddress(sR, sC)
+            end   = cellAddress(eR - 1, eC - 1)
             if start == end:
-                return f"{sheet.title}!{start}"
+                return f"{worksheet.title}!{start}"
         except:
             self.logger.exception('x')
             pass
         return None
 
-    def reloadNamedRanges(self):
+    def retrieveNamedRanges(self):
         self.logger.debug('Reloading GSheet(%s) named ranges' % self.name)
         self._timeoutRefreshNamedRanges=self.timeout(180)
 
         gs=self.gsheet()
+        gs.retrieveWorksheets()
         spreadsheet=gs.spreadsheet()
         items=spreadsheet.list_named_ranges()
         # self.logger.warning(items)
@@ -562,109 +610,25 @@ class MBIOTaskGSheet(MBIOTask):
                 for nr in items:
                     name=nr.get('name').lower()
                     gr=nr.get('range')
+                    # sheetId is 0 if not given
                     sid=gr.get('sheetId', 0)
-                    sheet=gs.sheet(sid)
-                    address=self.gridRangeToCellAddress(sheet, gr)
+                    worksheet=gs.worksheet(sid)
+                    address=self.gridRangeToCellAddress(worksheet, gr)
                     if name and address:
                         self._namedRanges[name]=address.lower()
 
                 self._timeoutRefreshNamedRanges=self.timeout(60)
             except:
+                self.logger.exception('x')
                 pass
 
-    """
-    def sheetExportManager(self) -> bool:
-        result=True
-
-        # collect all cells with pending sync
-        addresses=[]
-
-        # prepare data that need to be sent (pushed) to the sheet
-        data=[]
-
-        # EXPORTS (key->cell maping without using mbio values)
-        for export in sheets['exports']:
-            self.microsleep()
-            if not self.isTimeout(export['timeout']):
-                continue
-
-            export['timeout']=self.timeout(export['period'])
-            address=export['cell']
-            if not address in addresses:
-                addresses.append(address)
-                key=export['key']
-                value=self.getMBIO().value(key)
-                if value is not None:
-                    data.append({'range': address, 'values': [[value.value]]})
-                    if export['storeunit']:
-                        row,col=gspread.utils.a1_to_rowcol(address)
-                        col+=1
-                        address=gspread.utils.rowcol_to_a1(row, col)
-                        data.append({'range': address, 'values': [[value.unitstr()]]})
-
-        if data:
-            # push the data in one request
-            self.logger.debug('Write cells %s!%s to GSheet(%s)' % (sheet['name'], addresses, self.name))
-
-            gs=self.gsheet()
-            result=gs.set(sheet['name'], data)
-
-            # inhibit the gsheet sync for a time
-            self._timeoutSync=self.timeout(5)
-
-            # force a value refresh after sync
-            self._timeoutRefresh=0
-
-            for name in cells:
-                cell=self.cell(name)
-                value=cell['value']
-                if result:
-                    value.clearSync()
-                    value.setError(False)
-                else:
-                    value.setError(True)
-
-        return result
-
-
-
-
-
-
-                override=cell['override']
-                try:
-                    v=self.str2value(data[n][0][0])
-                    if v is not None:
-                        value.updateValue(v)
-                        value.setError(False)
-                        if override is not None:
-                            ovalue=self.getMBIO().value(override)
-                            if ovalue is not None:
-                                ovalue.manual(v)
-                        if cell['retain']:
-                            self.pickleWrite('cell-%s-retain' % cell['name'], v)
-                        continue
-                except:
-                    pass
-
-                if override is not None:
-                    ovalue=self.getMBIO().value(override)
-                    if ovalue is not None:
-                        ovalue.auto()
-
-    """
-
     def sheetImportManager(self, sheet) -> bool:
         result=True
 
         for item in sheet['imports']:
-            address=self.addressFromNamedRange(item['cell'])
-            if self.isA1Notation(address):
-                address='%s!%s' % (sheet['name'], address)
-            v=self._addresses.get(address)
-
+            v=self.getAdressValue(sheet, item['cell'])
             value=self.getMBIO().value(item['key'])
-            self.logger.warning("IMPORT %s:%s --> %s" % (address, v, value))
+            # self.logger.warning("IMPORT %s-->%s" % (v, value))
 
             if value is not None:
                 if v is not None:
@@ -675,87 +639,69 @@ class MBIOTaskGSheet(MBIOTask):
 
         return result
 
-    def sheetSyncManager(self, sheet) -> bool:
-        result=True
+    def a1_to_rowcol(self, address):
+        if address:
+            try:
+                address=self.addressFromNamedRange(address)
+                if '!' in address:
+                    address=address.split('!')[1]
+                row,col=gspread.utils.a1_to_rowcol(address)
+                return row, col
+            except:
+                pass
+        return None
 
-        self._timeoutSync=self.timeout(1)
+    def rowcol_to_a1(self, row, col):
+        try:
+            return gspread.utils.rowcol_to_a1(row, col)
+        except:
+            pass
+
+    def sheetExportManager(self, sheet):
+        gs=self.gsheet()
+
+        if not gs.worksheet(sheet['name']):
+            self.logger.debug('GSheet(%s) ignore offline sheet %s EXPORTs' % (self.name, sheet['name']))
+            return
+
+        # EXPORTS (key->cell maping without using mbio values)
+        for export in sheet['exports']:
+            self.microsleep()
+            address=self.addressFromNamedRange(export['xaddress'])
+            key=export['key']
+            value=self.getMBIO().value(key)
+            if value is not None and value.value is not None:
+                v=self.getAdressValue(sheet, export['cell'])
+                if v is None or value.checkIfValueWillTriggerNotify(v):
+                    self.logger.debug('GSheet(%s) EXPORT %s->%s (%s)' % (self.name, value, address, v))
+                    data=[value.value]
+                    if export['storeunit']:
+                        data.append(value.unitstr())
+                    gs.bufferedWrite(address, data)
+
+    def sheetSyncManager(self, sheet):
+        gs=self.gsheet()
 
         # collect all cells with pending sync
-        addresses=[]
         cells=sheet['writablecells']
 
-        if not cells:
-            return True
-
-        # prepare data that need to be sent (pushed) to the sheet
-        data=[]
-
         # AO, DO
         for name in cells:
             self.microsleep()
             cell=self.cell(name)
             value=cell['value']
-            self.evalValueExpression(value)
 
             if value.isPendingSync():
-                addresses.append(cell['address'])
-                data.append({'range': cell['address'], 'values': [[value.toReachValue]]})
+                address=self.addressFromNamedRange(cell['xaddress'])
+                data=[value.toReachValue]
                 if cell['storeunit']:
-                    row,col=gspread.utils.a1_to_rowcol(cell['address'])
-                    col+=1
-                    address=gspread.utils.rowcol_to_a1(row, col)
-                    data.append({'range': address, 'values': [[value.unitstr()]]})
-
-        """
-        # EXPORTS (key->cell maping without using mbio values)
-        for export in sheets['exports']:
-            self.microsleep()
-            if not self.isTimeout(export['timeout']):
-                continue
-
-            export['timeout']=self.timeout(export['period'])
-            address=export['cell']
-            if not address in addresses:
-                addresses.append(address)
-                key=export['key']
-                value=self.getMBIO().value(key)
-                if value is not None:
-                    data.append({'range': address, 'values': [[value.value]]})
-                    if export['storeunit']:
-                        row,col=gspread.utils.a1_to_rowcol(address)
-                        col+=1
-                        address=gspread.utils.rowcol_to_a1(row, col)
-                        data.append({'range': address, 'values': [[value.unitstr()]]})
-        """
-
-        if data:
-            # push the data in one request
-            self.logger.debug('Write cells %s!%s to GSheet(%s)' % (sheet['name'], addresses, self.name))
-
-            gs=self.gsheet()
-            result=gs.set(sheet['name'], data)
-
-            # inhibit the gsheet sync for a time
-            if result:
-                self._timeoutSync=self.timeout(5)
-            else:
-                self._timeoutSync=self.timeout(15)
-
-            # force a value refresh after sync
-            self._timeoutRefresh=0
-
-            for name in cells:
-                cell=self.cell(name)
-                value=cell['value']
-                if result:
-                    value.clearSync()
+                    data.append(value.unitstr())
+                if gs.bufferedWrite(address, data):
                     value.setError(False)
                 else:
                     value.setError(True)
+                value.clearSync()
 
-        return result
-
-    # TODO: to be integrated in syncSheetManager (allowing using only one request)
     def sheetDumpManager(self, sheet):
         gs=self.gsheet()
         for dump in sheet['dumps']:
@@ -764,67 +710,64 @@ class MBIOTaskGSheet(MBIOTask):
                 continue
 
             values=self.getMBIO().values(dump['key'])
-            address=dump['cell']
-            row,col=gspread.utils.a1_to_rowcol(address)
+            address=self.normalizeAddress(sheet, dump['cell'])
+            self.logger.debug('Dumping %s(%s) from GSheet(%s) to %s' % (sheet['name'], dump['key'], self.name, address))
+            row,col=self.a1_to_rowcol(address)
             if values:
-                data=[]
                 for value in values:
                     iotag=''
                     if value.config.iomaptag:
                         iotag=value.config.iomaptag
-                    data.append({'range': address, 'values': [[value.key, iotag, value.value, value.unitstr(), value.flags]]})
+
+                    gs.bufferedWrite(address, [value.key, iotag, value.value, value.unitstr(), value.flags])
                     row+=1
-                    address=gspread.utils.rowcol_to_a1(row, col)
-                if data:
-                    self.logger.debug('Dumping %s(%s) from GSheet(%s) to %s' % (sheet['name'], dump['key'], self.name, dump['cell']))
-                    gs.set(sheet['name'], data)
+                    address=self.rowcol_to_a1(row, col)
+                    address=self.normalizeAddress(sheet, address)
                     dump['timeout']=self.timeout(dump['refresh'])
 
-    def sheetManager(self, sheet) -> bool:
-        result=True
-
-        # process sheet "imports"
-        if sheet['imports']:
-            self.microsleep()
-            self.sheetImportManager(sheet)
-
-        # process sheet "dumps"
-        if sheet['dumps']:
-            self.microsleep()
-            self.sheetDumpManager(sheet)
-
-        return result
-
     def run(self):
-        result=True
+        gs=self.gsheet()
 
         if self.isTimeout(self._timeoutRefreshNamedRanges):
-            self.reloadNamedRanges()
-
-        if self.isTimeout(self._timeoutSync):
-            for sheet in self._sheets.values():
-                self.microsleep()
-                if not self.sheetSyncManager(sheet):
-                    result=False
+            self.retrieveNamedRanges()
+            self.logger.debug(self._namedRanges)
 
         # refresh spreadsheet cells values (read values from spreadsheet)
-        if self.config.refreshperiod>0 and self.isTimeout(self._timeoutRefresh):
+        if self.isTimeout(self._timeoutRefresh):
             self._timeoutRefresh=self.timeout(self.config.refreshperiod)
 
             if self._addresses:
-                gs=self.gsheet()
-                addresses=[x for x in self._addresses.keys()]
-                self.logger.debug('Retrieve cells %s from GSheet(%s)' % (addresses, self.name))
-                data=gs.batchget(addresses)
-                if data:
-                    # reset values
-                    for address in self._addresses:
-                        self._addresses[address]=None
+                addresses=[]
+                for address in self._addresses.keys():
+                    # self.logger.debug(address)
+                    address=self.addressFromNamedRange(address)
 
-                    # self.logger.warning(data)
-                    for address in data:
-                        v=self.str2value(data[address])
-                        self._addresses[address]=v
+                    if address is None:
+                        # self.logger.warning("IGNORE refresh for unknown address %s" % address)
+                        continue
+
+                    # WARNING!
+                    # we have to avoid request a non existing sheet (batchget will globally fails and return nothing)
+                    worksheet=self.worksheetFromAddress(address)
+                    if worksheet is None:
+                        self.logger.warning("GSheet(%s) IGNORE refresh address %s (OFFLINE)" % (self.name, address))
+                        continue
+
+                    addresses.append(address)
+
+                if addresses:
+                    self.logger.debug('Retrieve cells %s from GSheet(%s)' % (addresses, self.name))
+                    data=gs.batchget(addresses)
+                    # self.logger.debug(data)
+                    if data:
+                        self.logger.debug(data)
+                        # reset values
+                        for address in self._addresses:
+                            self._addresses[address]=None
+
+                        for address in data:
+                            v=self.str2value(data[address])
+                            self._addresses[address]=v
 
             # update values
             for cell in self._cells.values():
@@ -836,28 +779,28 @@ class MBIOTaskGSheet(MBIOTask):
                     value.updateValue(v)
                     value.setError(False)
                     if cell['retain']:
-                        self.pickleWrite('cell-%s-retain' % cell['name'], v)
+                        self.pickleWrite('%s-cell-retain' % cell['name'], v)
                     continue
 
                 value.setError(True)
-                result=False
 
         for sheet in self._sheets.values():
             self.microsleep()
-            if not self.sheetManager(sheet):
-                result=False
+            self.sheetImportManager(sheet)
+            self.sheetDumpManager(sheet)
+            self.sheetSyncManager(sheet)
+            self.sheetExportManager(sheet)
+
+        if self.isTimeout(self._timeoutCommit):
+            if gs.isBufferedWrite():
+                self.logger.info('Commit GSheet(%s) %s' % (self.name, gs._bufferWrite))
+                if not gs.commit():
+                    self.logger.error('Commit GSheet(%s) %s ERROR' % (self.name, gs._bufferWrite))
+
+                self._timeoutCommit=self.timeout(5)
 
-        # FIXME: not accurate way to do it
-        self.values.comerr.updateValue(not result)
         return 5.0
 
     @property
     def sid(self):
         return self.config.id
-
-    def setSheetId(self, sid):
-        if sid:
-            self.logger.info('Remap sheet id to %s...' % sid)
-            self.pickleWrite('id', sid)
-            self.config.id=sid
-            self.reset()
