@@ -681,6 +681,17 @@ class MBIO(object):
             values=sorted(values, key=lambda value: value.key)
         return values
 
+    def markValues(self, key='*', state=True):
+        count=0
+        for value in self.values(key):
+            count+=1
+            if count % 32 == 0:
+                self.microsleep()
+            value.mark(state)
+
+    def unmarkValue(self, key='*'):
+        self.markValues(key, False)
+
     def topvalues(self, key='*'):
         """Return the MBIOValues given by it's key using value.match() sorted by number of notifyCount"""
         values=[]
